import pg from 'pg'
import fs from 'fs'

const { Client } = pg

const getClient = async () => {
    const client = new Client({
        user: process.env.POSTGRES_USER,
        password: process.env.POSTGRES_PASSWORD,
        host: "postgresql",
        port: 5432,
        database: process.env.POSTGRES_DB
    });
    await client.connect()

    await client.query(fs.readFileSync('./scheme.psql').toString())

    return client
}

export const db = await getClient()


const getAmountOfUserPictures = async (roomId) => {
    return (await db.query("SELECT cardinality(pictures_urls) AS length FROM users WHERE room_id = $1", [roomId])).rows[0].length;
}

const getCurrentUserAction = async (roomId) => {
    return (await db.query('SELECT current_action FROM users WHERE room_id = $1', [roomId])).rows[0].current_action;
}

const setUserState = async (roomId, state) => {
    await db.query("UPDATE users SET current_action = $1 WHERE room_id = $2", [state, roomId]);
}

const appendUserPictures = async (roomId, mxc) => {
    await db.query("UPDATE users SET pictures_urls = array_append(pictures_urls, $1) WHERE room_id = $2 RETURNING cardinality(pictures_urls) AS amount", [mxc, roomId]);
}

const eraseUser = async (roomId) => {
    await db.query("DELETE FROM users WHERE room_id = $1", [roomId]);
}

const selectProfilesForUser = async (roomId) => {

    let userAge = (await db.query("SELECT age FROM users WHERE room_id = $1", [roomId])).rows[0].age
    //Selecting profiles other than user's and with difference in age +-2.
    return (await db.query("SELECT room_id, name, age, sex, description, country, city, pictures_urls FROM users WHERE age::numeric <@ ANY(ARRAY[numrange($1 - 2, $1 + 2)]) AND room_id != $2 ORDER BY RANDOM() LIMIT 1", [userAge, roomId])).rows[0]
}

const setUserCurrentlyViewingProfile = async (roomId, anotherRoomId) => {
    await db.query("UPDATE users SET currently_viewing = $1 WHERE room_id = $2", [anotherRoomId, roomId]);
}

const getUserCurrentlyViewingProfile = async (roomId) => {
    return (await db.query("SELECT currently_viewing FROM users WHERE room_id = $1", [roomId])).rows[0].currently_viewing
}

//Newlike is a room id of a user who was liked_profiles
const appendUserLikes = async (roomId, newLike) => {
    await db.query("UPDATE users SET liked_profiles = array_append(liked_profiles, $1) WHERE room_id = $2", [newLike, roomId])
    await db.query("UPDATE users SET likes_from = array_append(likes_from, $1) WHERE room_id = $2", [roomId, newLike])
}

const getUserLikes = async (roomId) => {
    return (await db.query("SELECT likes FROM users WHERE room_id = $1", [roomId])).rows[0].likes
}

const checkForMutualLike = async (roomId1, roomId2) => {
    return (await db.query("SELECT EXISTS (SELECT 1 FROM users u1 JOIN users u2 ON u1.room_id = ANY(u2.liked_profiles) AND u2.room_id = ANY(u1.liked_profiles) WHERE u1.room_id = $1 AND u2.room_id = $2 );", [roomId1, roomId2])).rows[0].exists
}

const getProfileInfo = async (roomId) => {
    return (await db.query("SELECT mx_id, room_id, name, age, sex, description, country, city, pictures_urls FROM users WHERE room_id = $1", [roomId])).rows[0];
}

const getAllLikesForUser = async (roomId) => {
    return (await db.query("SELECT room_id FROM users WHERE $1 = ANY(liked_profiles)", [roomId])).rows
}

const removeLikeFromUser = async (roomId, liked) => {
    await db.query("UPDATE users SET liked_profiles = ARRAY_REMOVE(liked_profiles, $1) WHERE room_id = $2", [roomId, liked])
}
export { eraseUser, appendUserPictures, setUserState, getCurrentUserAction, getAmountOfUserPictures, selectProfilesForUser, setUserCurrentlyViewingProfile, getUserCurrentlyViewingProfile, appendUserLikes, getUserLikes, checkForMutualLike, getProfileInfo, getAllLikesForUser, removeLikeFromUser }