import {
    logError,
    logInfo,
    readConfig,
    readMessages
} from './utils.js';

import { db, setUserState, selectProfilesForUser, setUserCurrentlyViewingProfile, getProfileInfo, getAllLikesForUser, removeLikeFromUser, checkForMutualLike } from "./db.js"

const messages = readMessages()

const requiresLengths = {
    "country": 64,
    "city": 64,
    "description": 512,
    "name": 32,
}

const processRequest = async (client, roomId, question, answer, nextQuestion) => {
    if (answer.length > requiresLengths[question]) {
        await client.sendText(roomId, messages.errors.toobig);
        return;
    }
    await db.query(`UPDATE users SET ${question} = $1 WHERE room_id = $2`, [answer, roomId]);
    await client.sendText(roomId, `Set your ${question} setting to "${answer}".`);
    await client.sendText(roomId, messages.setup[nextQuestion]); //next question
    setUserState(roomId, nextQuestion);
}

const showRandomProfileToUser = async (client, roomId) => {
    let chosenProfile = await selectProfilesForUser(roomId);
    if (!chosenProfile) {
        await client.sendText(roomId, messages.errors.noprofiles);
        return;
    }
    let message =
        `${chosenProfile.country}, ${chosenProfile.city}.
${chosenProfile.name}, ${chosenProfile.sex == 'm' ? 'male' : 'female'}, ${chosenProfile.age}.
${chosenProfile.description}`

    await setUserCurrentlyViewingProfile(roomId, chosenProfile.room_id)
    await client.sendText(roomId, message);

    if (chosenProfile.pictures_urls) {
        for (let picture of chosenProfile.pictures_urls) {
            console.log(picture)
            await client.sendMessage(roomId, {
                msgtype: "m.image",
                body: "image.png",
                url: picture
            });
        }
    }


    await client.sendText(roomId, messages.general.rate)
}

const showProfileToUser = async (client, roomId, profileId) => {
    let profileInfo = await getProfileInfo(profileId);
    console.log(profileInfo)
    let message =
        `${profileInfo.country}, ${profileInfo.city}.
${profileInfo.name}, ${profileInfo.sex == 'm' ? 'male' : 'female'}, ${profileInfo.age}.
${profileInfo.description}`

    await client.sendText(roomId, messages.general.showalike);
    await client.sendText(roomId, message);

    if (profileInfo.pictures_urls) {
        for (let picture of profileInfo.pictures_urls) {
            console.log(picture)
            await client.sendMessage(roomId, {
                msgtype: "m.image",
                body: "image.png",
                url: picture
            });
        }
    }

    await client.sendText(roomId, messages.general.mxid + profileInfo.mx_id)
}

const showNewLikes = async (client, roomId) => {
    let likes = (await getAllLikesForUser(roomId));
    console.log(`likes`);
    console.log(likes)
    for (let liked of likes) {
        // if (await checkForMutualLike(liked.roomId, roomId)) {
        await showProfileToUser(client, roomId, liked.room_id);
        await removeLikeFromUser(roomId, liked.room_id);
        // }
    }
}

export { processRequest, showRandomProfileToUser, showNewLikes, showProfileToUser };